!
! Copyright (C) 2013 A. Dal Corso 
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
SUBROUTINE set_hamiltonian(xk,ecut,npw)
!
!  This routine sets the free electron Hamiltonian. 
!
USE kinds, ONLY   : dp
USE freemod, ONLY   : g, et, ngm, nbnd
!
!  +++ USE here freemod to pass the hamiltonian eigenvalues and eigenvectors
!
IMPLICIT NONE

REAL(DP), INTENT(IN) :: xk(3), ecut
INTEGER, INTENT(OUT) :: npw

REAL(DP), ALLOCATABLE :: xkpg2(:)
INTEGER,  ALLOCATABLE :: igk(:)

INTEGER :: ig
REAL(DP) :: xmod, xkpg(3)

ALLOCATE(xkpg2(ngm))
ALLOCATE(igk(ngm))
!
! ++++ Compute all k+G within a sphere of radius sqrt(ecut) 
!      call mpw the number of plane waves and set their modulus in xkpg2
!
npw=0
DO ig=1,ngm
   xkpg(:) = xk(:) + g(:,ig)
   xmod= xkpg(1) ** 2 + xkpg(2) ** 2 + xkpg(3) ** 2
   IF (xmod <= ecut) THEN
      npw=npw+1
      xkpg2(npw) = xmod
      igk(npw)= ig    ! note the difference with the free electron, now
                      ! igk tell us the G vector for each k+G in the list of 
                      ! G
   ENDIF
ENDDO

IF (npw < nbnd) THEN
   write(6,*) "Too few plane waves, increase cut-off"
   stop 1
ENDIF 
!
!  +++ here allocate space for the Hamiltonian, the eigenvectors and
!      the eigenvalues
!


!
!  ++++ implement here the calculation of the Hamiltonian
!
DEALLOCATE(xkpg2)
DEALLOCATE(igk)

RETURN
END SUBROUTINE set_hamiltonian

